@@ -4,8 +4,8 @@
 
 *********************************************************************
  File		: 	$RCSfile: eciadsl-pppoeci.c,v $
- Version	:	$Revision: 1.1 $
- Modified by	:	$Author: flashcode $ ($Date: 2004/04/23 01:30:45 $)
+ Version	:	$Revision: 1.14 $
+ Modified by	:	$Author: kolja_gava $ ($Date: 2005/06/29 08:20:45 $)
  Licence	:	GPL
 *********************************************************************
 
@@ -83,6 +83,17 @@
 	Antoine REVERSAT aka Crevetor 4/1/2004 : These last days : got rid of linux/ * includes (thanks to MD)
 		   				 Added support for 2.6 kernels (binary stays the same but works for both kernels)
 		   				 Changed pusb_set_interface_alt to allow alt setting 0 to be used (which wasn't the case before)
+
+    21/11/2003 Kolja (gavaatbergamoblog.it)
+    Enanchement for gs7470 chipset support
+  
+  	21/02/2004 Kolja (gavaatbergamoblog.it)
+  	Make process works also with altInterface 0 for GS7470 modem chipset
+  	 - this change involve the EP_DATA_IN process :
+  	 		the EP_DATA_IN number became  0x83 and it uses Interrupt instead ISOCH.
+
+	29/04/2005 Kolja (gavaatbergamoblog.it)
+	Add new eoc handlig process defined in eci-common .
 */
 
 #define _GNU_SOURCE
@@ -116,16 +127,23 @@
 #include "util.h"
 /* my USB library */
 #include "pusb.h"
-/* GS 7070 control routines */
-#include "gs7070.h"
+/* GS interface control routines */
+#include "gsinterface.h"
+/* eoc handling interface */
+#include "eci-common/eoc.h"
+/* msg que handler*/
+#include "ecimsgh.h"
 
 #define LOG_FILE "/tmp/eciadsl-pppoeci.log"
 
 #define PPP_BUF_SIZE 64*1024
 
+struct eci_msg ecimsg;
+
 char* exec_filename;
 
 extern struct config_t config;
+extern struct eci_device_t eci_device;
 
 typedef enum
 {
@@ -156,6 +174,12 @@ const char* mode_name[] =
 
 encapsulation_mode default_frame_type=VCM_RFC2364;
 
+#ifdef USE_BIG_ENDIAN
+#define HDLC_HEADER (short)0xff03
+#else
+#define HDLC_HEADER (short)0x03ff
+#endif
+
 const char* frame_header[] =
 {
 	"",											/* VCM_RFC2364 */
@@ -203,26 +227,16 @@ typedef enum
 #define ERR_BUFSIZE (1024 - 1)
 char errText[ERR_BUFSIZE + 1];
 
-#define PAD_SIZE 11
 /* for debugging only */
 void dump_urb(char* buffer, int size);
 
 /* for ident(1) command */
-static const char id[] = "@(#) $Id: eciadsl-pppoeci.c,v 1.1 2004/04/23 01:30:45 flashcode Exp $";
+static const char id[] = "@(#) $Id: eciadsl-pppoeci.c,v 1.14 2005/06/29 08:20:45 kolja_gava Exp $";
 
 /* USB level */
 #define DATA_TIMEOUT 300
 int data_timeout = 0;
 
-#include "modem.h"
-
-/* max size of data endpoint (Windows driver uses 448). */
-#define MAX_EP_SIZE 448
-/* number of simultaneous URB submitted to data endpoint (Windows drv : 20 */
-#define NB_PKT_EP_DATA_IN 20
-/* number of ISO packet per URB (Windows driver uses 10) */
-#define PKT_NB 10
-
 /* ATM level : adapt according your ADSL provider settings */
 unsigned int my_vpi = 0xffffffff;
 unsigned int my_vci = 0xffffffff;
@@ -231,10 +245,7 @@ unsigned int vendor = 0;
 unsigned int product = 0;
 
 int pusb_set_interface_alt = -1;
-
-#define CELL_SIZE 53
-#define CELL_HDR  05
-#define CELL_DATA 48
+char* chipset="GSNONE";
 
 int syncHDLC = 1;
 
@@ -242,17 +253,18 @@ int syncHDLC = 1;
 
 /* global parameters */
 int verbose = 0;
-int usb_sync = 1;
 pusb_device_t fdusb;
-/* unsigned char lbuf[20][4400]; */
 pusb_endpoint_t ep_data_out, ep_data_in, ep_int;
 pid_t this_process;	 /* always the current process pid */
+int use_DataIn_ISO_Urb = 1; /* define if must use ISO for DATA_IN endpoint (default) otherwise it uses Interrupt*/
 
 /* predeclarations */
 
-int aal5_read(unsigned char* cell_buf, /* size_t count, */
-				unsigned int vpi, unsigned int vci,
-				unsigned int pti, int fdout);
+int aal5_read(const unsigned char* cell_buf, /* size_t count, */
+              unsigned int vpi, unsigned int vci,
+              unsigned int pti, int fdout);
+
+int init_ep_data_in_int(pusb_endpoint_t ep_data_in);
 
 /*
  * variable that will be used in several function
@@ -383,7 +395,7 @@ void fatal_error(const error_codes err, const char* text)
 	else \
 		printf(".");
 
-void dump(unsigned char* buf, int len)
+void dump(const unsigned char* buf, int len)
 {
 	int i, j;
 
@@ -705,26 +717,25 @@ int ppp_write(int fd, unsigned char* buf, int n)
 }
 
 /*
- * cell_read : is used to decode an individual cell and to try to make
- * an AAL5 frame.
+
+ cell_read: decode an individual ATM cell and to try to make an AAL5
+ frame. return value: -2 if vpi.vci is incorrect, 0 if everything is OK, <0 on
+ other errors
+
  */
 
-int cell_read(unsigned char* cellbuf, /* size_t count, */ int fdout)
-{
-	unsigned int vpi, vci, pti;
+int cell_read(const unsigned char* cellbuf, /* size_t count, */ int fdout){
+	struct aal5_header_st aal5Header;
 
-	/* decoding & printing of the cell */
-	vpi = ( cellbuf[0]         <<  4) | (cellbuf[1] >> 4);
-	vci = ((cellbuf[1] & 0x0f) << 12) | (cellbuf[2] << 4) | (cellbuf[3] >> 4);
-	pti = ( cellbuf[3] & 0x0f);
+	getAal5HeaderStructure(cellbuf, &aal5Header);
 
-	if (vpi != my_vpi || vci != my_vci)
+	if (aal5Header.vpi != my_vpi || aal5Header.vci != my_vci)
 	{
 		/* there's some junk so try to re-sync */
 		return(-2);
 	}
 	/* filling our AAL5 frame */
-	return(aal5_read(cellbuf + CELL_HDR, /* CELL_DATA, */ vpi, vci, pti, fdout));
+	return(aal5_read(cellbuf + eci_device.cell_r_hdr, /* CELL_DATA, */ aal5Header.vpi, aal5Header.vci, aal5Header.pti, fdout));
 }
 
 int cell_write(unsigned char* cellbuf, unsigned char* buf, int n,
@@ -733,7 +744,7 @@ int cell_write(unsigned char* cellbuf, unsigned char* buf, int n,
 	unsigned char gfc = 0;
 	unsigned char hec = 0x00; /* change to the value used in the Windows driver */
 
-	if (n != CELL_DATA)
+	if (n != eci_device.cell_w_data)
 		return(-1);
 
 	/* normalize */
@@ -747,10 +758,10 @@ int cell_write(unsigned char* cellbuf, unsigned char* buf, int n,
 	cellbuf[3] = ((vci & 0xf) << 4) | pti;
 	cellbuf[4] = hec;
 
-	memcpy(cellbuf + CELL_HDR,  buf, CELL_DATA);
-	memset(cellbuf + CELL_SIZE, 0xff, PAD_SIZE);
+	memcpy(cellbuf + eci_device.cell_w_hdr,  buf, eci_device.cell_w_data);
+	memset(cellbuf + eci_device.cell_w_size, 0xff, eci_device.pad_size);
 
-	return(CELL_SIZE + PAD_SIZE);
+	return(eci_device.cell_w_size + eci_device.pad_size);
 }
 
 /*
@@ -759,7 +770,7 @@ int cell_write(unsigned char* cellbuf, unsigned char* buf, int n,
  * (0 otherwise)
  */
 
-int aal5_read(unsigned char* cell_buf, /* size_t count, */
+int aal5_read(const unsigned char* cell_buf, /* size_t count, */
 		unsigned int vpi, unsigned int vci, unsigned int pti, int fdout)
 {
 	static unsigned char buf[64 * 1024];
@@ -777,7 +788,7 @@ int aal5_read(unsigned char* cell_buf, /* size_t count, */
 					"vpi.vci mismatch (%d.%d instead of %d.%d) => ignored",
 					 vpi, vci, my_vpi, my_vci);
 			message(errText);
-			dump(cell_buf, CELL_DATA);
+			dump(cell_buf, eci_device.cell_r_data);
 		}
 		return(-2); /* return -2 and ? */
 	}
@@ -787,10 +798,10 @@ int aal5_read(unsigned char* cell_buf, /* size_t count, */
 	 * copying the cell's content to our buffer
 	 */
 
-	if ((unsigned int)(len + CELL_DATA) < sizeof(buf))
+	if ((unsigned int)(len + eci_device.cell_r_data) < sizeof(buf))
 	{
-		memcpy(buf + len, cell_buf, CELL_DATA);
-		len += CELL_DATA;
+		memcpy(buf + len, cell_buf, eci_device.cell_r_data);
+		len += eci_device.cell_r_data;
 	}
 	else
 	{
@@ -870,7 +881,7 @@ int aal5_read(unsigned char* cell_buf, /* size_t count, */
 int aal5_write(pusb_endpoint_t epdata, unsigned char* buf, int n)
 {
 	/* add the trailer */
-	int total_len = 48 * ((n + 8 + CELL_DATA - 1) / 48);
+	int total_len = 48 * ((n + 8 + eci_device.cell_w_data - 1) / 48);
 	unsigned int crc, pti = 0;
 
 	/*
@@ -902,17 +913,17 @@ int aal5_write(pusb_endpoint_t epdata, unsigned char* buf, int n)
 	while (total_len > 0)
 	{
 
-		if (total_len <= CELL_DATA)
+		if (total_len <= eci_device.cell_w_data)
 			/* this is the last cell, so mark it */
 			pti = 2;
 
 		ptr += cell_write(bigbuf + ptr, buf,
-				CELL_DATA, my_vpi, my_vci, pti);
+				eci_device.cell_w_data, my_vpi, my_vci, pti);
 
 		/* send the same cell over another vpi/vci */
 
-		buf += CELL_DATA;
-		total_len -= CELL_DATA;
+		buf += eci_device.cell_w_data;
+		total_len -= eci_device.cell_w_data;
 
 	}
 
@@ -961,7 +972,7 @@ int aal5_write(pusb_endpoint_t epdata, unsigned char* buf, int n)
 }
 /* returns -1 in case of errors */
 
-int decode_usb_pkt(unsigned char* buf, int len)
+int decode_usb_pkt(const unsigned char* buf, int len)
 {
 	static unsigned char rbuf [64 * 1024];
 	static int rlen = 0;
@@ -975,12 +986,12 @@ int decode_usb_pkt(unsigned char* buf, int len)
 		snprintf(errText, ERR_BUFSIZE,
 				"reading from USB: len=%d", len);
 		message(errText);
-		dump(buf, len);
+        dump(buf, len);
 	}
 
 	/* add the received buf to our buf, if possible */
 
-	if ((unsigned int)(rlen + len) < sizeof(rbuf))
+	if (rlen + len < (int)sizeof(rbuf))
 	{
 		memcpy(rbuf + rlen, buf, len);
 		rlen += len;
@@ -993,7 +1004,7 @@ int decode_usb_pkt(unsigned char* buf, int len)
 	}
 
 	pos = 0;
-	while ((rlen - pos) >= CELL_SIZE)
+	while ((rlen - pos) >= eci_device.cell_r_size)
 	{
 		if ((r = cell_read(rbuf + pos, /* CELL_SIZE, */ gfdout)) < 0 && r != -2)
 		{
@@ -1007,10 +1018,11 @@ int decode_usb_pkt(unsigned char* buf, int len)
 			block header (just add 1 onto pos!!) for now!! */
 			pos++;
 		else
-			pos += CELL_SIZE;
+			pos += eci_device.cell_r_size;
 	}
-	rlen -= pos;
 
+    /* adjust the buffer (rbuf, rlen) */
+	rlen -= pos;
 	if (rlen != 0)
 		memcpy(rbuf, rbuf + pos, rlen);
 
@@ -1035,6 +1047,14 @@ int init_ep_int(pusb_endpoint_t ep_int)
 
 	int i, ret;
 
+/* setup new oec handling - kolja */
+#if 0
+	/* allocate variables for GS interface - kolja */
+	allocateGSint();
+#else
+	eoc_init();
+#endif
+
 	for (i = 0; i < NB_PKT_EP_INT; i++)
 	{
 		/* we use a BULK transfer instead, since INTERRUPT transfer
@@ -1057,14 +1077,50 @@ int init_ep_int(pusb_endpoint_t ep_int)
 
 int init_ep_data_in(pusb_endpoint_t ep_data_in)
 {
-	static unsigned char buf[NB_PKT_EP_DATA_IN][MAX_EP_SIZE * PKT_NB];
-
+	unsigned char* buf; 
 	int i, ret;
 
-	for (i = 0; i < NB_PKT_EP_DATA_IN; i++)
+    buf = (unsigned char *) malloc(
+        eci_device.eci_nb_pkt_ep_data_in * eci_device.eci_nb_iso_packet
+        * eci_device.eci_iso_packet_size);
+
+	/* if DATA_IN type is not ISO call init for Interrupt handling - kolja*/
+	if(!use_DataIn_ISO_Urb)
+		return(init_ep_data_in_int(ep_data_in));
+	
+	for (i = 0; i < eci_device.eci_nb_pkt_ep_data_in; i++)
 	{
-		ret = pusb_endpoint_submit_iso_read(ep_data_in, buf[i], MAX_EP_SIZE,
-						    PKT_NB, 0); /* SIGRTMIN); */
+		ret = pusb_endpoint_submit_iso_read(ep_data_in, buf+( eci_device.eci_nb_pkt_ep_data_in* i), eci_device.eci_iso_packet_size,
+						    eci_device.eci_nb_iso_packet, 0); /* SIGRTMIN); */
+		if (ret < 0)
+		{
+			snprintf(errText, ERR_BUFSIZE,
+					"error on initial submit of URB %d on EP_DATA_IN", i);
+			message(errText);
+			perror("error: init_ep_data_in");
+			return(ret);
+		}
+	}
+
+	return(0);
+}
+
+/* handle initialisation of DATA_IN type Interrupt - kolja*/
+int init_ep_data_in_int(pusb_endpoint_t ep_data_in)
+{
+
+	unsigned char* buff;
+	int i, ret;
+
+	buff = (unsigned char*) malloc(eci_device.eci_nb_pkt_ep_data_in * 0x40);
+
+	for (i = 0; i < eci_device.eci_nb_pkt_ep_data_in; i++)
+	{
+		/* we use a BULK transfer instead, since INTERRUPT transfer
+		   are not supported by usbdevfs (devio.c)
+		*/
+		ret = pusb_endpoint_submit_read(ep_data_in, buff + (eci_device.eci_nb_pkt_ep_data_in * i),
+						    0x40, 0); /* SIGRTMIN); */
 		if (ret < 0)
 		{
 			snprintf(errText, ERR_BUFSIZE,
@@ -1080,9 +1136,6 @@ int init_ep_data_in(pusb_endpoint_t ep_data_in)
 
 void handle_ep_int(unsigned char* buf, int size, pusb_device_t fdusb)
 {
-	static int lost_synchro = 0;
-	static GS7070int* gs = 0;
-
 	unsigned char outbuf[40] =
 	{
 		0xff, 0xff, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
@@ -1091,12 +1144,12 @@ void handle_ep_int(unsigned char* buf, int size, pusb_device_t fdusb)
 		0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
 		0x0c, 0x0c
 	};
+	
+#if 0
+	static int lost_synchro = 0;
 	int i, outi = 0;
 	int dataBlock = 0;
 
-	if (gs==0)
-		gs=allocateGS7070int();
-
 	if (verbose > 1)
 	{
 		snprintf(errText, ERR_BUFSIZE,
@@ -1107,46 +1160,43 @@ void handle_ep_int(unsigned char* buf, int size, pusb_device_t fdusb)
 
 	if (!lost_synchro || 1)
 	{
-		for (i = 3; i < 18; i++)
+		for (i = eci_device.ep_int_data_start_point; i < 18; i++)
 		{
 			unsigned short code = (buf[2 * i + 0] << 8) | buf[2 * i + 1];
 
 			if (code != 0x0c0c)
 			{ /* don't bother with 0c0c blocks, there 'empty' */
 				dataBlock=-1;
-				gs7070SetControl(gs, buf + i * 2);
+				gsSetControl(buf + i * 2);
+				memcpy(outbuf + 10 + outi, buf + i * 2, 2);
+				gsGetResponse(outbuf + 10 + outi);
+				outi = outi+2;				
 			}
 		}
 	}
 	if (dataBlock == 0)
 		/* no data/control information so don't bother to respond */
 		return;
-
-	for (i = 0; i < 15; i++)
+#else
+	if (verbose > 1)
 	{
-		unsigned char b1, b2;
-
-		b1 = buf[6 + 2 * i + 0];
-		b2 = buf[6 + 2 * i + 1];
-
-		/* each word != 0x0c 0x0c is copied in the output buffer */
-
-		if (b1 != 0x0c || b2 != 0x0c)
-		{
-			if ((unsigned int)(10 + outi) >= sizeof(outbuf))
-			{
-				message("warning: outbuf is too small");
-				dump(buf, size);
-				break;
-			}
-			outbuf[10 + outi] = b1;
-			outbuf[10 + outi+1] = b2;
-			gs7070GetResponse(gs, outbuf + 10 + outi);
-			outi += 2;
-		}
+		snprintf(errText, ERR_BUFSIZE,
+				"reading from ep int: len=%d", size);
+		message(errText);
+		dump(buf, size);
 	}
 
-	if (pusb_control_msg(fdusb, 0x40, 0xdd, 0xc02, 0x580, outbuf,
+	if(buf[0]!=0xf0)
+		return;
+	parse_eoc_buffer(buf+(eci_device.ep_int_data_start_point+1), eci_device.ep_int_data_size);
+	
+	if (has_eocs() == 0)
+		/* no data/control information so don't bother to respond */
+		return;
+	get_eoc_answer(outbuf+8);		
+#endif
+
+	if (pusb_control_msg(fdusb, 0x40, 0xdd, eci_device.bulk_response_value, 0x580, outbuf,
 			sizeof(outbuf), DATA_TIMEOUT) != sizeof(outbuf))
 	{
 		message("error on sending vendor device URB");
@@ -1174,12 +1224,11 @@ void handle_urb(pusb_urb_t urb)
 	unsigned char* buf;
 	unsigned char* sbuf;
 	int idx, size, ret;
+	int ep;
 
-	switch (pusb_urb_get_epnum(urb))
-	{
-		case EP_INT:
-			/* printf("interrupt!\n"); */
+	ep = pusb_urb_get_epnum(urb);
 
+	if (ep==eci_device.eci_int_ep){
 			if (pusb_urb_buffer_first(urb, &buf, &size, &idx))
 			{
 				do
@@ -1203,11 +1252,9 @@ void handle_urb(pusb_urb_t urb)
 					perror("error: can't re-submit on EP_INT");
 				}
 			}
-			break;
-
-		case EP_DATA_IN:
+	}else
+	if (ep==eci_device.eci_iso_ep){
 			/* printf("data in!\n"); */
-
 			if (pusb_urb_buffer_first(urb, &buf, &size, &idx))
 			{
 				sbuf = buf;
@@ -1217,7 +1264,7 @@ void handle_urb(pusb_urb_t urb)
 					if (size != 0)
 					{
 /*						printf("data_in %02x, packet len = %d\n",
-							   EP_DATA_IN, size);
+							   epnum_data_in_used, size);
 						dump(buf, size);
 */
 						handle_ep_data_in(buf, size);
@@ -1225,28 +1272,87 @@ void handle_urb(pusb_urb_t urb)
 				}
 				while (pusb_urb_buffer_next(urb, &buf, &size, &idx));
 
-				ret=pusb_endpoint_submit_iso_read(ep_data_in, sbuf, MAX_EP_SIZE,
-								  PKT_NB, 0); /* SIGRTMIN); */
-				if (ret < 0)
-				{
+			/* use ISO URB depending on AltInterface Setted - kolja*/
+			ret=pusb_endpoint_submit_iso_read(ep_data_in, sbuf, eci_device.eci_iso_packet_size, eci_device.eci_nb_iso_packet, 0); /* SIGRTMIN); */
+			if (ret < 0){
 					message("error on re-submit URB on EP_DATA_IN");
 					perror("error: can't re-submit on ep EP_DATA_IN");
 				}
 			}
-			break;
+	}else
+	if (ep==eci_device.eci_in2_ep){
+			if (pusb_urb_buffer_first(urb, &buf, &size, &idx)){
+			sbuf = buf;
 
-		case EP_DATA_OUT:
-			/* ok, nothing to say */
-			break;
+			do{
+				if (size != 0){
+/*						printf("data_in %02x, packet len = %d\n",
+							   EP_DATA_IN2, size);
+						dump(buf, size);
+*/
+					handle_ep_data_in(buf, size);
+				}
+			} while (pusb_urb_buffer_next(urb, &buf, &size, &idx));
 
-		default:
+			/* Interrupt URB depending on AltInterface Setted - kolja*/
+			ret = pusb_endpoint_submit_read(ep_data_in, buf, 0x40, 0); /* SIGRTMIN); */					
+			if (ret < 0){
+				message("error on re-submit URB on EP_DATA_IN2");
+				perror("error: can't re-submit on ep EP_DATA_IN2");
+			}
+		}
+	}else
+	if (ep!=eci_device.eci_bulk_ep){
 			snprintf(errText, ERR_BUFSIZE,
 					"Error: unknown endpoint : %02x", pusb_urb_get_epnum(urb));
 			message(errText);
-			break;
 	}
 }
 
+/* this function check for message and do the appropriate action */
+int ecimsgh(void){
+	int ret=0;
+	int i;
+	unsigned char outbuf[40] =
+	{
+		0xff, 0xff, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
+		0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
+		0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
+		0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c, 0x0c,  0x0c,
+		0x0c, 0x0c
+	};
+	ret = rcvEciMsg(&ecimsg,0,0);
+	if(ret>0){
+		switch(ecimsg.ecicmd){
+			case ECI_MC_DO_DISCONNECT:
+				get_eoc_answer_DISCONNECT(outbuf+8);
+				for(i=0;i<5;i++){
+					if (pusb_control_msg(fdusb, 0x40, 0xdd, eci_device.bulk_response_value, 0x580, outbuf,
+							sizeof(outbuf), DATA_TIMEOUT) != sizeof(outbuf))
+					{
+						message("error on sending vendor device URB");
+						perror("error: can't send vendor device!");
+					}
+					else
+					{
+						if (verbose > 1)
+						{
+							snprintf(errText, ERR_BUFSIZE,
+							"ctrl msg sent: len=%d", sizeof(outbuf));
+							message(errText);
+							dump(outbuf, sizeof(outbuf));
+						}
+					}
+				}
+				sndEciMsg(ECI_MC_DISCONNECTED, NULL, 0, ecimsg.sender, 1);
+				return(0);				
+				break;			
+		}
+	}
+	return(1);
+}
+
+
 /*
   handle_ep_data_in_ep_int : must be called after init_ep_int()
   and init_ep_data_int()
@@ -1257,10 +1363,12 @@ void handle_ep_data_in_ep_int(/* pusb_endpoint_t ep_data_in,
 {
 	pusb_urb_t urb;
 
-	for (;;)
+	/* initialize message queue handler */
+	initEciMsgQueue(ECI_PT_PPP);
+	while(ecimsgh() && !has_eoc_problem())
 	{
 		urb = pusb_device_get_urb(fdusb);
-		if (urb == NULL)
+		if ((urb == NULL) || (pusb_get_urb_status(urb)<0))
 			break;
 
 		handle_urb(urb);
@@ -1274,7 +1382,7 @@ void handle_ep_data_in_ep_int(/* pusb_endpoint_t ep_data_in,
 
 		free(urb);
 	}
-
+	deallocEciMsgQueue();
 	message("end of handle_ep_data_in_ep_int");
 }
 
@@ -1311,12 +1419,12 @@ void handle_ep_data_out(pusb_endpoint_t epdata, int fdin)
 					"aal5_write=%d", r);
 			message(errText);
 			tcount++;
-			if (tcount > 3)
+			if (tcount > 1)
 			    break;
 		}
 	    else
 			if (tcount > 0)
-				tcount--;
+				tcount=0;
 	}
 
 	snprintf(errText, ERR_BUFSIZE,
@@ -1338,7 +1446,7 @@ void* fn_handle_ep_data_out(void* ignored)
 
 void version(const int full)
 {
-	printf("%s, part of " PRODUCT_NAME " (" PRODUCT_ID ") " PRODUCT_VERSION " (" __DATE__ " " __TIME__ ")\n",
+	printf("%s, part of " PACKAGE_NAME PACKAGE_VERSION " (" __DATE__ " " __TIME__ ")\n",
 			exec_filename);
 	if (full)
 		printf("%s\n", id);
@@ -1372,9 +1480,9 @@ void usage(const int ret)
 					"\n");
 	fprintf(stderr, "vpi value must be between 0 and %u\n", 0xfff);
 	fprintf(stderr, "vci value must be between 0 and %u\n", 0xffff);
-	fprintf(stderr, "If ALL parameters but switches are omitted, the ones from " CONF_PATH CONF_DIR "/eciadsl.conf\n"
+	fprintf(stderr, "If ALL parameters but switches are omitted, the ones from %s\n"
 					"are assumed.\n"
-					"\n");
+					"\n", config_filename());
 	fprintf(stderr,	"The vpi and vci are numerical values. They define the vpi.vci used\n"
 					"by provider. For instance: 8.35 for France, 0.38 for UK.\n"
 					"\n");
@@ -1470,6 +1578,9 @@ int main(int argc, char** argv)
 		if ((strcmp(argv[i], "-alt") == 0) && (i + 1 < argc))
 			get_signed_value(argv[++i], &pusb_set_interface_alt);
 		else
+		if (((strcmp(argv[i], "-mc") == 0) || (strcmp(argv[i], "--modem_chipset") == 0) ) && (i + 1 < argc))
+			strDup(&chipset, argv[++i]);
+		else				
 		if ((strcmp(argv[i], "-dto") == 0) && (i + 1 < argc))
 			get_signed_value(argv[++i], &data_timeout);
 		else
@@ -1507,7 +1618,6 @@ int main(int argc, char** argv)
 
 	/* read the configuration file */
 	read_config_file();
-
 	/* try to assume default values from the config file */
 	if ((my_vci == 0xffffffff) && (config.vci != 0xffffffff))
 			my_vci = config.vci;
@@ -1555,12 +1665,15 @@ int main(int argc, char** argv)
 		usage(-1);
 	}
 
+	if (strcmp(chipset,"GSNONE")!=0){
+		set_eci_modem_chipset(chipset);
+	}
+
 	/* check some other params */
 	if (pusb_set_interface_alt < 0)
 	{
-		fprintf(stderr, "incorrect alt interface value (%d underflows)\n",
-			pusb_set_interface_alt);
-		usage(-1);
+		/* set dafault interface - kolja*/ 
+		pusb_set_interface_alt = config.alt_interface_synch;
 	}
 	if (data_timeout < 0)
 	{
@@ -1817,8 +1930,13 @@ int main(int argc, char** argv)
 	/* Initialize global variables */
 	gfdout = fdout;
 
-	/* set interface */
+	/* we claim interface 0 (altsetting != 0 (we use 4)), where endpoints 0x02, 0x86 & 0x88 are */
 
+	if (pusb_claim_interface(fdusb, 0) < 0)
+		fatal_error(ERR_PUSB_CLAIM_INTERFACE,
+					"pusb_claim_interface 0 failed");
+
+	/* set interface */
 	if (pusb_set_interface_alt > -1)
 		if (pusb_set_interface(fdusb, 0, pusb_set_interface_alt) < 0)
 		{
@@ -1827,18 +1945,22 @@ int main(int argc, char** argv)
 			fatal_error(ERR_PUSB_SET_INTERFACE, errText);
 		}
 
-	/* we claim interface 0 (altsetting != 0 (we use 4)), where endpoints 0x02, 0x86 & 0x88 are */
+	/* if altInterface is setted to 0 use Interrupt instead ISO for DATA_IN - kolja*/
+	if (pusb_set_interface_alt == 0){
+		use_DataIn_ISO_Urb = 0;
+	}
 
-	if (pusb_claim_interface(fdusb, 0) < 0)
-		fatal_error(ERR_PUSB_CLAIM_INTERFACE,
-					"pusb_claim_interface 0 failed");
-
-	ep_int = pusb_endpoint_open(fdusb, EP_INT, O_RDWR);
+	ep_int = pusb_endpoint_open(fdusb, eci_device.eci_int_ep, O_RDWR);
 	if (ep_int == NULL)
 		fatal_error(ERR_PUSB_OPEN_EP_INT,
 					"pusb_endpoint_open EP_INT failed");
 
-	ep_data_in = pusb_endpoint_open(fdusb, EP_DATA_IN, O_RDWR);
+	/* if no ISO used for DATA_IN endpoint number must be 0x83 - kolja */
+	if (use_DataIn_ISO_Urb)
+		ep_data_in = pusb_endpoint_open(fdusb, eci_device.eci_iso_ep, O_RDWR);
+	else
+		ep_data_in = pusb_endpoint_open(fdusb, eci_device.eci_in2_ep, O_RDWR);
+	
 	if (ep_data_in == NULL)
 		fatal_error(ERR_PUSB_OPEN_EP_DATA_IN,
 					"pusb_endpoint_open EP_DATA_IN failed");
@@ -1848,7 +1970,7 @@ int main(int argc, char** argv)
 	 * is an infinite loop and will return only on errors.
 	 */
 
-	ep_data_out = pusb_endpoint_open(fdusb, EP_DATA_OUT, O_RDWR);
+	ep_data_out = pusb_endpoint_open(fdusb, eci_device.eci_bulk_ep, O_RDWR);
 	if (ep_data_out == NULL)
 		fatal_error(ERR_PUSB_OPEN_EP_DATA_OUT,
 					"pusb_endpoint_open EP_DATA_OUT failed");
@@ -1871,9 +1993,8 @@ int main(int argc, char** argv)
 		_exit(0);
 	}
 #endif
-
+	pthread_t th_id;
 	{
-		pthread_t th_id;
 		pthread_attr_t attr;
 
 		pthread_attr_init(&attr);
@@ -1897,7 +2018,10 @@ int main(int argc, char** argv)
 	/* we release all the interface we'd claim before exiting */
 	if (pusb_release_interface(fdusb, 0) < 0)
 		message("pusb_release_interface failed");
-
+	
+	/* kill thread handling data out */
+	kill (th_id,SIGTERM);
+	
 	pusb_endpoint_close(ep_int);
 	pusb_endpoint_close(ep_data_in);
 	pusb_endpoint_close(ep_data_out);
